@@ -1,22 +1,22 @@
-import threading
-from util.threadinglocal import local as threading_local
-import sys
-import re
-import warnings
 import atexit
-import os
+from cgi import parse_qsl
+import inspect
 import new
+import os
+import sys
+import threading
 import types
 import urllib
+import warnings
 import weakref
-import inspect
-import sqlbuilder
+
 from cache import CacheSet
-import col
-import main
-from joins import sorter
-from converters import sqlrepr
 import classregistry
+import col
+from converters import sqlrepr
+import main
+import sqlbuilder
+from util.threadinglocal import local as threading_local
 
 warnings.filterwarnings("ignore", "DB-API extension cursor.lastrowid used")
 
@@ -38,7 +38,7 @@ class ConsoleWriter:
             try:
                 text = text.encode(self.dbEncoding)
             except UnicodeEncodeError:
-                text = repr(text)[1:-1]
+                text = repr(text)[2:-1] # Remove u'...' from the repr
         logfile.write(text + '\n')
 
 class LogWriter:
@@ -84,13 +84,12 @@ class DBConnection:
         self._connectionCount = 1
         self.autoCommit = Boolean(autoCommit)
         self.registry = registry or None
-        classregistry.registry(self.registry).addCallback(
-            self.soClassAdded)
+        classregistry.registry(self.registry).addCallback(self.soClassAdded)
         registerConnectionInstance(self)
         atexit.register(_closeConnection, weakref.ref(self))
 
-    def uri(self):
-        auth = getattr(self, 'user', '')
+    def oldUri(self):
+        auth = getattr(self, 'user', '') or ''
         if auth:
             if self.password:
                 auth = auth + ':' + self.password
@@ -101,17 +100,44 @@ class DBConnection:
         uri = '%s://%s' % (self.dbName, auth)
         if self.host:
             uri += self.host
+            if self.port:
+                uri += ':%d' % self.port
         uri += '/'
         db = self.db
         if db.startswith('/'):
-            db = path[1:]
+            db = db[1:]
         return uri + db
 
+    def uri(self):
+        auth = getattr(self, 'user', '') or ''
+        if auth:
+            auth = urllib.quote(auth)
+            if self.password:
+                auth = auth + ':' + urllib.quote(self.password)
+            auth = auth + '@'
+        else:
+            assert not getattr(self, 'password', None), (
+                'URIs cannot express passwords without usernames')
+        uri = '%s://%s' % (self.dbName, auth)
+        if self.host:
+            uri += self.host
+            if self.port:
+                uri += ':%d' % self.port
+        uri += '/'
+        db = self.db
+        if db.startswith('/'):
+            db = db[1:]
+        return uri + urllib.quote(db)
+
+    def connectionFromOldURI(cls, uri):
+        return cls._connectionFromParams(*cls._parseOldURI(uri))
+    connectionFromOldURI = classmethod(connectionFromOldURI)
+
     def connectionFromURI(cls, uri):
-        raise NotImplemented
+        return cls._connectionFromParams(*cls._parseURI(uri))
     connectionFromURI = classmethod(connectionFromURI)
 
-    def _parseURI(uri):
+    def _parseOldURI(uri):
         schema, rest = uri.split(':', 1)
         assert rest.startswith('/'), "URIs must start with scheme:/ -- you did not include a / (in %r)" % rest
         if rest.startswith('/') and not rest.startswith('//'):
@@ -159,6 +185,45 @@ class DBConnection:
                 argvalue = urllib.unquote(argvalue)
                 args[argname] = argvalue
         return user, password, host, port, path, args
+    _parseOldURI = staticmethod(_parseOldURI)
+
+    def _parseURI(uri):
+        protocol, request = urllib.splittype(uri)
+        user, password, port = None, None, None
+        host, path = urllib.splithost(request)
+
+        if host:
+            # Python < 2.7 have a problem - splituser() calls unquote() too early
+            #user, host = urllib.splituser(host)
+            if '@' in host:
+                user, host = host.split('@', 1)
+            if user:
+                user, password = [x and urllib.unquote(x) or None for x in urllib.splitpasswd(user)]
+            host, port = urllib.splitport(host)
+            if port: port = int(port)
+        elif host == '':
+            host = None
+
+        # hash-tag is splitted but ignored
+        path, tag = urllib.splittag(path)
+        path, query = urllib.splitquery(path)
+
+        path = urllib.unquote(path)
+        if (os.name == 'nt') and (len(path) > 2):
+            # Preserve backward compatibility with URIs like /C|/path;
+            # replace '|' by ':'
+            if path[2] == '|':
+                path = "%s:%s" % (path[0:2], path[3:])
+            # Remove leading slash
+            if (path[0] == '/') and (path[2] == ':'):
+                path = path[1:]
+
+        args = {}
+        if query:
+            for name, value in parse_qsl(query):
+                args[name] = value
+
+        return user, password, host, port, path, args
     _parseURI = staticmethod(_parseURI)
 
     def soClassAdded(self, soClass):
@@ -582,7 +647,7 @@ class DBAPI(DBConnection):
             if key in kw:
                 value = kw.pop(key)
                 if col.from_python:
-                    value = col.from_python(value, soClass)
+                    value = col.from_python(value, sqlbuilder.SQLObjectState(soClass, connection=self))
                 data[col.dbName] = value
             elif col.foreignName in kw:
                 obj = kw.pop(col.foreignName)
@@ -926,7 +991,7 @@ class ConnectionURIOpener(object):
             assert inst.name.find(':') == -1, "You cannot include ':' in your class names (%r)" % cls.name
             self.instanceNames[inst.name] = inst
 
-    def connectionForURI(self, uri, **args):
+    def connectionForURI(self, uri, oldUri=False, **args):
         if args:
             if '?' not in uri:
                 uri += '?' + urllib.urlencode(args)
@@ -937,7 +1002,10 @@ class ConnectionURIOpener(object):
         if uri.find(':') != -1:
             scheme, rest = uri.split(':', 1)
             connCls = self.dbConnectionForScheme(scheme)
-            conn = connCls.connectionFromURI(uri)
+            if oldUri:
+                conn = connCls.connectionFromOldURI(uri)
+            else:
+                conn = connCls.connectionFromURI(uri)
         else:
             # We just have a name, not a URI
             assert self.instanceNames.has_key(uri), \
@@ -966,5 +1034,6 @@ import maxdb
 import mssql
 import mysql
 import postgres
+import rdbhost
 import sqlite
 import sybase
