@@ -1,12 +1,14 @@
 #!/usr/bin/env python
-import optparse
+
 import fnmatch
-import re
+import optparse
 import os
+import re
 import sys
 import textwrap
-import warnings
 import time
+import warnings
+
 try:
     from paste.deploy import appconfig
 except ImportError:
@@ -14,9 +16,9 @@ except ImportError:
 
 import sqlobject
 from sqlobject import col
-from sqlobject.util import moduleloader
-from sqlobject.declarative import DeclarativeMeta
 from sqlobject.classregistry import findClass
+from sqlobject.declarative import DeclarativeMeta
+from sqlobject.util import moduleloader
 
 # It's not very unsafe to use tempnam like we are doing:
 warnings.filterwarnings(
@@ -193,7 +195,7 @@ class Command(object):
         # having to modify any core SQLObject component and namespace
         # contamination.
         # yemartin - 2006-08-08
-        
+
         class SQLObjectCircularReferenceError(Exception): pass
 
         def findReverseDependencies(cls):
@@ -206,10 +208,10 @@ class Command(object):
                 if col.foreignKey:
                     other = findClass(col.foreignKey,
                                       col.soClass.sqlmeta.registry)
-                    if other not in depended:
+                    if (other is not cls) and (other not in depended):
                         depended.append(other)
             return depended
-        
+
         # Cache to save already calculated dependency levels.
         dependency_levels = {}
         def calculateDependencyLevel(cls, dependency_stack=[]):
@@ -237,7 +239,7 @@ class Command(object):
                 level = 0
             dependency_levels[cls] = level
             return level
-        
+
         # Now simply calculate and sort by dependency levels:
         try:
             sorter = []
@@ -689,9 +691,7 @@ class CommandDrop(Command):
         v = self.options.verbose
         dropped = 0
         not_existing = 0
-        rev_classes = self.classes()
-        rev_classes.reverse()
-        for soClass in rev_classes:
+        for soClass in reversed(self.classes()):
             exists = soClass._connection.tableExists(soClass.sqlmeta.table)
             if v >= 1:
                 if exists:
@@ -876,7 +876,7 @@ class CommandRecord(Command):
     help = ('Record state of table definitions.  The state of each '
             'table is written out to a separate file in a directory, '
             'and that directory forms a "version".  A table is also '
-            'added to you datebase (%s) that reflects the version the '
+            'added to your database (%s) that reflects the version the '
             'database is currently at.  Use the upgrade command to '
             'sync databases with code.'
             % SQLObjectVersionTable.sqlmeta.table)
@@ -1258,7 +1258,7 @@ class CommandUpgrade(CommandRecord):
         return upgraders[-1]
 
 def update_sys_path(paths, verbose):
-    if isinstance(paths, (str, unicode)):
+    if isinstance(paths, basestring):
         paths = [paths]
     for path in paths:
         path = os.path.abspath(path)
