@@ -1,7 +1,9 @@
-import sqlbuilder
 import dbconnection
-import main
 import joins
+import main
+import sqlbuilder
+
+__all__ = ['SelectResults']
 
 class SelectResults(object):
     IterationClass = dbconnection.Iteration
@@ -152,8 +154,8 @@ class SelectResults(object):
                         end = start
                     else:
                         end = value.stop + self.ops.get('start', 0)
-                        if self.ops.get('end', None) is not None \
-                           and value['end'] < end:
+                        if self.ops.get('end', None) is not None and \
+                                self.ops['end'] < end:
                             # truncated by previous slice:
                             end = self.ops['end']
                 else:
@@ -229,11 +231,15 @@ class SelectResults(object):
             or a dot-q attribute (like Table.q.aColumn)
         """
         expressions = []
+        conn = self._getConnection()
+        if self.ops.get('distinct'):
+            distinct = 'DISTINCT '
+        else:
+            distinct = ''
         for func_name, attribute in attributes:
-            if isinstance(attribute, str):
-                expression = '%s(%s)' % (func_name, attribute)
-            else:
-                expression = getattr(sqlbuilder.func, func_name)(attribute)
+            if not isinstance(attribute, str):
+                attribute = conn.sqlrepr(attribute)
+            expression = '%s(%s%s)' % (func_name, distinct, attribute)
             expressions.append(expression)
         return self.accumulate(*expressions)
 
@@ -338,5 +344,3 @@ class SelectResults(object):
         clause = sqlbuilder.AND(otherClass.q.id == getattr(intTable, join.otherColumn),
                      getattr(intTable, colName) == query.q.id)
         return otherClass, clause
-
-__all__ = ['SelectResults']
